from typing import Optional

from openstack.exceptions import ResourceNotFound

from interface_openstack.interface.network.FloatingIP import floating_ip
from interface_openstack.interface.network.FloatingIPManager import floating_ip_manager
from interface_openstack.implementation.utils.connection import OpenStackConnection


class OpenstackFloatingIPManager(FloatingIPManager, OpenStackConnection):

    @staticmethod
    def os_floating_ip_to_rc_floating_ip(os_floating_ip) -> FloatingIP:
        return FloatingIP(
            os_floating_ip.id,
            os_floating_ip.router_id,
            os_floating_ip.floating_network_id,
            os_floating_ip.subnet_id,
            os_floating_ip.port_id,
            os_floating_ip.name,
            os_floating_ip.fixed_ip_address,
            os_floating_ip.floating_ip_address,
            os_floating_ip.status,
            os_floating_ip.created_at,
            os_floating_ip.updated_at
        )

    def create(self, floating_network_id) -> FloatingIP:
        os_floating_ip = self.openstack.network.create_ip(floating_network_id=floating_network_id)

        return self.os_floating_ip_to_rc_floating_ip(os_floating_ip)

    def get(self, id) -> Optional[FloatingIP]:
        try:
            os_floating_ip = self.openstack.network.get_ip(id)
        except ResourceNotFound:
            return None

        return self.os_floating_ip_to_rc_floating_ip(os_floating_ip)

    def update(self, id, params) -> Optional[FloatingIP]:
        try:
            os_floating_ip = self.openstack.network.update_ip(id, **params)
        except ResourceNotFound:
            return None

        return self.os_floating_ip_to_rc_floating_ip(os_floating_ip)

    def delete(self, id) -> bool:
        try:
            self.openstack.network.delete_ip(id)
        except ResourceNotFound:
            return False

        return True

    def list(self) -> []:
        floating_ips = []

        for os_floating_ip in self.openstack.floating_ip.ips():
            floating_ips.append(self.os_floating_ip_to_rc_floating_ip(os_floating_ip))

        return floating_ips
